@@ -23,15 +23,12 @@ import os, sys, struct, operator
 import math, numarray,optparse, FFT
 import ossaudiodev, audioop
 
-__version__ = "$Revision: 1.2 $"
+__version__ = "$Revision: 1.3 $"
 __author__ = "Arnau Sanchez <arnau@ehas.org>"
 __depends__ = ['OSSAudioDev', 'FFT', 'Numeric-Extension', 'Python-2.4']
 __copyright__ = """Copyright (C) 2006 Arnau Sanchez <arnau@ehas.org>.
 This code is distributed under the terms of the GNU General Public License."""
 
-## TODO: 
-## 	Test new decode without FFT
-
 ### Global variables
 
 # DTMF tones
@@ -45,16 +42,16 @@ BUSY_FREQS = (480.0, 620.0)
 DIALTONE_FREQS = (350.0, 440.0)
 
 # Decoding constants
-MIN_RATE= 8000
-OVERLAPPING = 0.75
+MIN_RATE = 8000
 MIN_TONETIME = 0.1
+SUBWINDOW = 2
 
-DEF_DECODE_OVERPOWER = 25
+DEF_DECODE_OVERPOWER = 30
 DEF_MIN_DIFF23_POWER = 15
 DEF_MAX_DIFF12_POWER = 20
 
-PEAK_UP = 2
-PEAK_DOWN = 0.5
+PEAK_UP = 1
+PEAK_DOWN = 2
 
 # Common audio formats from ossaudiodev: 
 # dictionary: string_format : bit_order, sample size, signed/unsigned
@@ -163,8 +160,7 @@ class Decoder_state:
 	def __init__(self, decoder):
 		self.key_state = dict([(x, 0) for x in get_dtmf_keys()])
 		self.current_key = None
-		self.min_time = MIN_TONETIME
-		self.min_peaks = int(1 / (1-decoder.overlapping)) + 1
+		self.min_peaks = SUBWINDOW
 
 ###################################################
 ###################################################
@@ -199,13 +195,18 @@ class Decoder:
 		d1 = [((f1, f2), freqs_to_key(f1, f2)) for f1 in DTMF_LOW_FREQS for f2 in DTMF_HIGH_FREQS]
 		d2 = [((f2, f1), freqs_to_key(f1, f2)) for f1 in DTMF_LOW_FREQS for f2 in DTMF_HIGH_FREQS]
 		self.freqs_to_key_dict = dict(d1 + d2)
-		self.overlapping = args.get("overlapping", OVERLAPPING)
-		self.windowsize = 2 ** int(math.log(self.samplerate * self.channels * self.sample_length * MIN_TONETIME, 2)) 
+		self.windowsize = int(MIN_TONETIME * self.samplerate / SUBWINDOW)
 		self.ds = Decoder_state(self)
+		
+		self.cosarray = {}
+		self.sinarray = {}
+		for freq in DTMF_FREQS:
+			self.sinarray[freq] = numarray.array([math.sin(2*math.pi*freq*x/self.samplerate) for x in range(0, self.windowsize)])
+			self.cosarray[freq] = numarray.array([math.cos(2*math.pi*freq*x/self.samplerate) for x in range(0, self.windowsize)])
+
 		self.debug("sampling rate: %d" %self.samplerate)
 		self.debug("channels: %d" %self.channels)
-		self.debug("window size: %d bytes" %self.windowsize)
-		self.debug("overlapping: %0.2f" %self.overlapping)
+		self.debug("window size: %d samples" %self.windowsize)
 		self.debug("min peaks decoding: %d" %self.ds.min_peaks)
 		self.debug("Freqs 1&2 overpower: %0.2f dB" %(10*math.log(self.decode_overpower, 10)))
 		self.debug("Freqs 1&2 max diff: %0.2f dB" %(10*math.log(self.max_diff12_power, 10)))
@@ -218,26 +219,17 @@ class Decoder:
 		debug("decoder - " + log)
 
 	#################################
-	def decoding_simple(self, fft):
-		output = []
-		
-		# Get power for each DTMF frequency
-		freq_power = {}
-		for freq in DTMF_FREQS:
-			ifreq = int(round((freq / self.samplerate) * self.windowsize))
-			power = pow(abs(fft[ifreq]), 2)			
-			freq_power[freq] = power / self.windowsize
-
+	def decoding_simple(self, freq_power):		
 		# Get 2 max frequencies
 		sorted_freqs = [(freq_power[freq], freq) for freq in freq_power]
 		sorted_freqs.sort()
-		sorted_freqs.reverse()		
+		sorted_freqs.reverse()	
 		f1power, f1 = sorted_freqs[0]
 		f2power, f2 = sorted_freqs[1]
 		f3power, f3 = sorted_freqs[2]
 		try: key_max = self.freqs_to_key_dict[(f1,f2)]
 		except: key_max = None
-		
+		self.debug("f1=%0.2f (%0.5f), f2=%0.2f (%0.5f), f3=%0.2f (%0.5f)" %(f1, f1power, f2, f2power, f3, f3power), 2)
 		if key_max:
 			# Calculate mean power (discard the 2 max-frequencies) and min acceptable power
 			mean_power = 0
@@ -248,32 +240,37 @@ class Decoder:
 
 			# Check if higher frequencies are greater than minimum acceptable power
 			if f2power > min_overpower and f1power < f2power * self.max_diff12_power and f2power > f3power * self.min_diff23_power:			
-				self.debug("* tone threshold start", 2)
-				self.debug("f2power > min_overpower -- %f > %f" %(f2power, min_overpower), 2)
-				self.debug("f1power < f2power * max_diff12 -- %f < %f" %(f1power, f2power * self.max_diff12_power), 2)
-				self.debug("f2power > f3power * min_diff23 -- %f > %f" %(f2power, f3power * self.min_diff23_power), 2)
-				self.debug("* tone threshold end", 2)
 				key_max = self.freqs_to_key_dict[(f1,f2)]
+				self.debug("key_max: %s. ** ok **" %key_max, 2)
 			else: 
+				self.debug("key_max: %s. ko" %key_max, 2)
 				key_max = None
+			self.debug("f2power > min_overpower -- %f > %f" %(f2power, min_overpower), 2)
+			self.debug("f1power < f2power * max_diff12 -- %f < %f" %(f1power, f2power * self.max_diff12_power), 2)
+			self.debug("f2power > f3power * min_diff23 -- %f > %f" %(f2power, f3power * self.min_diff23_power), 2)
+
+		output = []
 
 		# Update state for each key. If min_peaks is reached for a key, append in DTMF output
+		if key_max or self.ds.current_key:
+			self.debug("%s-%s" %(key_max, self.ds.current_key))
 		for key in get_dtmf_keys():
 			old_state = self.ds.key_state[key]
 			if key == key_max:
-				if old_state < self.ds.min_peaks:
+				if self.ds.key_state[key] < self.ds.min_peaks:
 					self.ds.key_state[key] += PEAK_UP
 				if self.ds.key_state[key] >= self.ds.min_peaks and self.ds.current_key != key:
+					for k in [k for k in get_dtmf_keys() if k != key]:
+						self.ds.key_state[k] = 0
 					output.append(key)
 					self.ds.current_key = key
-					for key in [k for k in get_dtmf_keys() if k != key]:
-						self.ds.key_state[key] = 0
 					break
-			elif old_state > 0:
-				if old_state == 1 and self.ds.current_key == key:
-					self.ds.current_key = None						
+			elif key == self.ds.current_key and old_state > 0:
+				self.debug("peak_down: %f" %self.ds.key_state[key])
 				self.ds.key_state[key] -= PEAK_DOWN
-		
+				if self.ds.key_state[key] <= 0:
+					self.ds.key_state[key] = 0
+					self.ds.current_key = None
 		return output
 
 	################################################
@@ -285,15 +282,15 @@ class Decoder:
 		if self.audio_offset:
 			abuffer -= self.audio_offset
 		self.input = numarray.concatenate([self.input, abuffer]).tolist()
-		
 		dtmf_output = []
 		while len(self.input) >= self.windowsize:
-			window = self.input[:self.windowsize]
-			index = int(self.windowsize * (1.0 - self.overlapping))
-			fft = FFT.real_fft(window)
+			window = numarray.array(self.input[:self.windowsize])
+			self.input = self.input[self.windowsize:]
+			fft = {}
+			for freq in DTMF_FREQS:
+				fft[freq] = (((self.sinarray[freq] * window)).sum())**2 + (((self.cosarray[freq] * window)).sum())**2
 			keys = self.decoding_simple(fft)
 			dtmf_output += keys
-			self.input = self.input[index:]
 		
 		return dtmf_output
 
@@ -307,13 +304,13 @@ def main():
 	parser = optparse.OptionParser(usage)
 	
 	parser.add_option('-v', '--verbose-level', default=0, dest='verbose', action="count", help='Increase verbose level')
-	parser.add_option('-s', '--sample-rate', dest='samplerate', default = 8000, metavar='SPS', type='int', help = 'Set sampling rate')
-	parser.add_option('-f', '--sample-format', dest='sampleformat', default = "S16_LE", metavar='AFMT_FORMAT', type='string', help = 'Set audio sample format')
-	parser.add_option('-c', '--audio-channels', dest='channels', default = 1, metavar='NUMBER', type='int', help = 'Set audio channels')
+	parser.add_option('-s', '--samplerate', dest='samplerate', default = 8000, metavar='SPS', type='int', help = 'Set sampling rate')
+	parser.add_option('-f', '--sampleformat', dest='sampleformat', default = "S16_LE", metavar='AFMT_FORMAT', type='string', help = 'Set audio sample format')
+	parser.add_option('-c', '--channels', dest='channels', default = 1, metavar='NUMBER', type='int', help = 'Set audio channels')
 	parser.add_option('-e', '--sensibility', dest='sensibility', default = "1.0", metavar='VALUE', type='float', help = 'Decoding sensibility (1.0 for normal)')
 	parser.add_option('-g', '--generate', dest='generate', default = "", metavar='KEYS, TONETIME, WAITTIME, GAIN', type='string', help = 'DTMF generator')
 	parser.add_option('-d', '--decode', dest='decode', default = False, action='store_true', help = 'DTMF decoder')
-	parser.add_option('-b', '--buffer-size', dest='buffersize', default = 1024, metavar = "BYTES", type = int, help = 'Buffer size for input/output')
+	parser.add_option('-b', '--buffersize', dest='buffersize', default = 1024, metavar = "BYTES", type = int, help = 'Buffer size for input/output')
 
 	options, args = parser.parse_args()
 	stdin, stdout = sys.stdin.fileno(), sys.stdout.fileno()
